@@ -16,6 +16,19 @@
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *                                                                         *
+ *   In addition, as a special exception, the copyright holders give       *
+ *   permission to link the code of portions of this program with the      *
+ *   OpenSSL library under certain conditions as described in each         *
+ *   individual source file, and distribute linked combinations            *
+ *   including the two.                                                    *
+ *   You must obey the GNU General Public License in all respects          *
+ *   for all of the code used other than OpenSSL.  If you modify           *
+ *   file(s) with this exception, you may extend this exception to your    *
+ *   version of the file(s), but you are not obligated to do so.  If you   *
+ *   do not wish to do so, delete this exception statement from your       *
+ *   version.  If you delete this exception statement from all source      *
+ *   files in the program, then also delete it here.                       *
  ***************************************************************************/
 
 #include "PdfDifferenceEncoding.h"
@@ -2204,18 +2217,18 @@ const PdfEncodingDifference & PdfEncodingDifference::operator=( const PdfEncodin
     return *this;
 }
 
-void PdfEncodingDifference::AddDifference( int nCode )
+void PdfEncodingDifference::AddDifference( int nCode, pdf_utf16be unicodeValue )
 {
+    pdf_utf16be inCodePoint = unicodeValue;
+
 #ifdef PODOFO_IS_LITTLE_ENDIAN
-    pdf_utf16be inCodePoint = ((nCode & 0xff00) >> 8) | ((nCode & 0xff) << 8);
-#else
-    pdf_utf16be inCodePoint = nCode;
+    inCodePoint = ((inCodePoint & 0xff00) >> 8) | ((inCodePoint & 0xff) << 8);
 #endif // PODOFO_IS_LITTLE_ENDIAN
 
-    this->AddDifference( nCode, PdfDifferenceEncoding::UnicodeIDToName( inCodePoint ) );
+    this->AddDifference( nCode, unicodeValue, PdfDifferenceEncoding::UnicodeIDToName( inCodePoint ) );
 }
 
-void PdfEncodingDifference::AddDifference( int nCode, const PdfName & rName )
+void PdfEncodingDifference::AddDifference( int nCode, pdf_utf16be unicodeValue, const PdfName & rName, bool bExplicitNames )
 {
     if( nCode > 255 || nCode < 0 ) 
     {
@@ -2225,7 +2238,22 @@ void PdfEncodingDifference::AddDifference( int nCode, const PdfName & rName )
     TDifference dif;
     dif.nCode        = nCode;
     dif.name         = rName;
-    dif.unicodeValue = 0;
+    // In type3 fonts, glyph names are explicit keys from the font's CharProcs
+    // dictionary, therefore they have no meaning.
+    // By setting unicodeValue to nCode, we keep PdfEncodingDifference::Contains
+    // from calling PdfDifferenceEncoding::NameToUnicodeID, which would return 0
+    // after looking it up in the unicode tables. This allows us, provided the
+    // font's encoding is unicode-compatible, to preserve the characters' codes
+    // in the process. This seems to be Adobe Reader's behaviour as well.
+    if (bExplicitNames) {
+#ifdef PODOFO_IS_LITTLE_ENDIAN
+        dif.unicodeValue = ((nCode & 0xff00) >> 8) | ((nCode & 0xff) << 8);
+#else
+        dif.unicodeValue = nCode;
+#endif // PODOFO_IS_LITTLE_ENDIAN
+    } else {
+        dif.unicodeValue = unicodeValue;
+    }
 
     std::pair<TIVecDifferences,TCIVecDifferences> it = 
         std::equal_range( m_vecDifferences.begin(), m_vecDifferences.end(), dif, DifferenceComparatorPredicate() );
@@ -2269,6 +2297,24 @@ bool PdfEncodingDifference::Contains( int nCode, PdfName & rName, pdf_utf16be &
     return false;
 }
 
+bool PdfEncodingDifference::ContainsUnicodeValue( pdf_utf16be unicodeValue, char &rValue ) const
+{
+#ifdef PODOFO_IS_LITTLE_ENDIAN
+	unicodeValue = ((unicodeValue & 0xff00) >> 8) | ((unicodeValue & 0xff) << 8);
+#endif // PODOFO_IS_LITTLE_ENDIAN
+
+	TCIVecDifferences it, end = m_vecDifferences.end();
+	for (it = m_vecDifferences.begin(); it != end; it++) {
+		pdf_utf16be uv = it->unicodeValue;
+		if (uv == unicodeValue) {
+			rValue = it->nCode;
+			return true;
+		}
+	}
+
+	return false;
+}
+
 void PdfEncodingDifference::ToArray( PdfArray & rArray )
 {
     pdf_int64 nLastCode = -2;
@@ -2336,7 +2382,7 @@ PdfDifferenceEncoding::PdfDifferenceEncoding( const PdfEncodingDifference & rDif
     Init();
 }
 
-PdfDifferenceEncoding::PdfDifferenceEncoding( PdfObject* pObject, bool bAutoDelete )
+PdfDifferenceEncoding::PdfDifferenceEncoding( PdfObject* pObject, bool bAutoDelete, bool bExplicitNames )
     : PdfEncoding( 0x00, 0xff ), PdfElement( NULL, pObject ),
       m_bAutoDelete( bAutoDelete )
 {
@@ -2362,7 +2408,7 @@ PdfDifferenceEncoding::PdfDifferenceEncoding( PdfObject* pObject, bool bAutoDele
         const PdfArray & rDifferences = this->GetObject()->GetIndirectKey( PdfName("Differences") )->GetArray();
         PdfArray::const_iterator it = rDifferences.begin();
 
-        long long curCode = -1;
+        pdf_int64 curCode = -1;
 
         while( it != rDifferences.end() ) 
         {
@@ -2370,7 +2416,7 @@ PdfDifferenceEncoding::PdfDifferenceEncoding( PdfObject* pObject, bool bAutoDele
                 curCode = (*it).GetNumber();
             else if( (*it).IsName() ) 
             {
-                m_differences.AddDifference( static_cast<int>(curCode), (*it).GetName() );
+                m_differences.AddDifference( static_cast<int>(curCode), 0, (*it).GetName(), bExplicitNames );
                 ++curCode;
             }
             
@@ -2547,8 +2593,7 @@ PdfString PdfDifferenceEncoding::ConvertToUnicode( const PdfString & rEncodedStr
     return ret;
 }
 
-PdfRefCountedBuffer PdfDifferenceEncoding::ConvertToEncoding( const PdfString & rString, 
-                                                    const PdfFont* pFont ) const
+PdfRefCountedBuffer PdfDifferenceEncoding::ConvertToEncoding( const PdfString & rString, const PdfFont* /*pFont*/ ) const
 {
     const PdfEncoding* pEncoding = GetBaseEncoding();
 
@@ -2558,6 +2603,8 @@ PdfRefCountedBuffer PdfDifferenceEncoding::ConvertToEncoding( const PdfString &
     if( rString.IsUnicode() )
     {
         lLen = rString.GetCharacterLength();
+        if( !lLen )
+             return PdfRefCountedBuffer();
         pszUtf16 = static_cast<pdf_utf16be*>(malloc(sizeof(pdf_utf16be)*lLen));
         if( !pszUtf16 )
         {
@@ -2570,6 +2617,8 @@ PdfRefCountedBuffer PdfDifferenceEncoding::ConvertToEncoding( const PdfString &
         // Only do a copy if we really have to
         PdfString str = rString.ToUnicode();
         lLen = str.GetCharacterLength();
+        if( !lLen )
+            return PdfRefCountedBuffer();
         pszUtf16 = static_cast<pdf_utf16be*>(malloc(sizeof(pdf_utf16be)*lLen));
         if( !pszUtf16 )
         {
@@ -2578,23 +2627,36 @@ PdfRefCountedBuffer PdfDifferenceEncoding::ConvertToEncoding( const PdfString &
         memcpy( pszUtf16, str.GetUnicode(), lLen * sizeof(pdf_utf16be) );
     }
 
+    char* pDest = static_cast<char*>(malloc( sizeof(char) * (lLen + 1) ));
+    if( !pDest ) 
+    {
+        PODOFO_RAISE_ERROR( ePdfError_OutOfMemory );
+    }
+
+    char *pCur = pDest;
+    long lNewLen = 0L;
+
     for( int i=0;i<lLen;i++ ) 
     {
         pdf_utf16be val = pszUtf16[i];
-#ifdef PODOFO_IS_LITTLE_ENDIAN
-        val = ((val & 0xff00) >> 8) | ((val & 0xff) << 8);
-#endif // PODOFO_IS_LITTLE_ENDIAN
 
-        PdfName     name;
-        pdf_utf16be value;
-        if( m_differences.Contains( static_cast<int>(val), name, value ) )
-            pszUtf16[i] = value;
+        if (!m_differences.ContainsUnicodeValue(val, *pCur))
+            *pCur = static_cast<const PdfSimpleEncoding *>(pEncoding)->GetUnicodeCharCode(val);
+        if( *pCur) // ignore 0 characters, as they cannot be converted to the current encoding
+        {
+            ++pCur; 
+            ++lNewLen;
+        }
     }
 
-    PdfString ret( pszUtf16, lLen );
+    *pCur = '\0';
+
+    PdfRefCountedBuffer cDest( lNewLen );
+    memcpy( cDest.GetBuffer(), pDest, lNewLen );
+    free( pDest );
     free( pszUtf16 );
 
-    return pEncoding->ConvertToEncoding( ret, pFont );
+    return cDest;
 }
 
 const PdfEncoding* PdfDifferenceEncoding::GetBaseEncoding() const
