@@ -16,14 +16,29 @@
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *                                                                         *
+ *   In addition, as a special exception, the copyright holders give       *
+ *   permission to link the code of portions of this program with the      *
+ *   OpenSSL library under certain conditions as described in each         *
+ *   individual source file, and distribute linked combinations            *
+ *   including the two.                                                    *
+ *   You must obey the GNU General Public License in all respects          *
+ *   for all of the code used other than OpenSSL.  If you modify           *
+ *   file(s) with this exception, you may extend this exception to your    *
+ *   version of the file(s), but you are not obligated to do so.  If you   *
+ *   do not wish to do so, delete this exception statement from your       *
+ *   version.  If you delete this exception statement from all source      *
+ *   files in the program, then also delete it here.                       *
  ***************************************************************************/
 
 #include "PdfImage.h"
 
 #include "base/PdfDefinesPrivate.h"
 
+#include "base/PdfArray.h"
 #include "base/PdfColor.h"
 #include "base/PdfStream.h"
+#include "base/PdfFiltersPrivate.h"
 
 #include <stdio.h>
 #include <wchar.h>
@@ -49,6 +64,7 @@ extern "C" {
 #  endif
 #  include "jpeglib.h"
 }
+
 #endif // PODOFO_HAVE_JPEG_LIB
 
 #ifdef PODOFO_HAVE_PNG_LIB
@@ -116,9 +132,18 @@ const char** PdfImage::GetSupportedFormats()
     return ppszFormats;
 }
 
-void PdfImage::SetImageColorSpace( EPdfColorSpace eColorSpace )
+void PdfImage::SetImageColorSpace( EPdfColorSpace eColorSpace, const PdfArray *indexedData )
 {
-    this->GetObject()->GetDictionary().AddKey( PdfName("ColorSpace"), ColorspaceToName( eColorSpace ) );
+    if (eColorSpace == ePdfColorSpace_Indexed) {
+        PODOFO_RAISE_LOGIC_IF( !indexedData, "PdfImage::SetImageColorSpace: indexedData cannot be NULL for Indexed color space." );
+
+        PdfArray array(*indexedData);
+
+        array.insert(array.begin(), ColorspaceToName( eColorSpace ));
+        this->GetObject()->GetDictionary().AddKey( PdfName("ColorSpace"), array );
+    } else {
+        this->GetObject()->GetDictionary().AddKey( PdfName("ColorSpace"), ColorspaceToName( eColorSpace ) );
+    }
 }
 
 void PdfImage::SetImageICCProfile( PdfInputStream* pStream, long lColorComponents, EPdfColorSpace eAlternateColorSpace ) 
@@ -229,10 +254,6 @@ void PdfImage::LoadFromFile( const char* pszFilename )
 }
 
 #ifdef PODOFO_HAVE_JPEG_LIB
-#if !defined(PODOFO_JPEG_RUNTIME_COMPATIBLE)
-void jpeg_memory_src (j_decompress_ptr cinfo, const JOCTET * buffer, size_t bufsize);
-#endif // PODOFO_JPEG_RUNTIME_COMPATIBLE
-
 extern "C" {
 static void JPegErrorExit(j_common_ptr cinfo)
 {
@@ -347,6 +368,72 @@ void PdfImage::LoadFromJpegHandle( PdfFileInputStream* pInStream )
     
     (void) jpeg_destroy_decompress(&cinfo);
 }
+
+void PdfImage::LoadFromJpegData(const unsigned char* pData, pdf_long dwLen)
+{
+    struct jpeg_decompress_struct cinfo;
+    struct jpeg_error_mgr         jerr;
+
+    cinfo.err = jpeg_std_error(&jerr);
+    jerr.error_exit = &JPegErrorExit;
+    jerr.emit_message = &JPegErrorOutput;
+
+    jpeg_create_decompress(&cinfo);
+
+    jpeg_memory_src(&cinfo, pData, dwLen);
+
+    if( jpeg_read_header(&cinfo, TRUE) <= 0 )
+    {
+        (void) jpeg_destroy_decompress(&cinfo);
+
+        PODOFO_RAISE_ERROR( ePdfError_UnexpectedEOF );
+    }
+
+    jpeg_start_decompress(&cinfo);
+
+    m_rRect.SetWidth( cinfo.output_width );
+    m_rRect.SetHeight( cinfo.output_height );
+
+    // I am not sure wether this switch is fully correct.
+    // it should handle all cases though.
+    // Index jpeg files might look strange as jpeglib+
+    // returns 1 for them.
+    switch( cinfo.output_components )
+    {
+        case 3:
+            this->SetImageColorSpace( ePdfColorSpace_DeviceRGB );
+            break;
+        case 4:
+	{
+	    this->SetImageColorSpace( ePdfColorSpace_DeviceCMYK );
+	    // The jpeg-doc ist not specific in this point, but cmyk's seem to be stored
+	    // in a inverted fashion. Fix by attaching a decode array
+	    PdfArray decode;
+	    decode.push_back( 1.0 );
+	    decode.push_back( 0.0 );
+	    decode.push_back( 1.0 );
+	    decode.push_back( 0.0 );
+	    decode.push_back( 1.0 );
+	    decode.push_back( 0.0 );
+	    decode.push_back( 1.0 );
+	    decode.push_back( 0.0 );
+	    
+	    this->GetObject()->GetDictionary().AddKey( PdfName("Decode"), decode );
+	}
+	break;
+        default:
+            this->SetImageColorSpace( ePdfColorSpace_DeviceGray );
+            break;
+    }
+    
+    // Set the filters key to DCTDecode
+    this->GetObject()->GetDictionary().AddKey( PdfName::KeyFilter, PdfName( "DCTDecode" ) );
+    
+    PdfMemoryInputStream fInpStream( (const char*)pData, (pdf_long) dwLen);
+    this->SetImageDataRaw( cinfo.output_width, cinfo.output_height, 8, &fInpStream );
+    
+    (void) jpeg_destroy_decompress(&cinfo);
+}
 #endif // PODOFO_HAVE_JPEG_LIB
 
 #ifdef PODOFO_HAVE_TIFF_LIB
